/*
 * Copyright (C) 2000, iAgora LLC.
 * Written by Roger Espel Llima for iAgora LLC, 28-NOV-2000.
 *
 * This program is Free Software.  You can redistribute and/or modify it
 * under the terms specified in the file ``LICENSE'', which is part of
 * the lingerd distribution.  (For the impatient: it's an Apache-style
 * license, without the advertising clause.)
 *
 */

/* 
 * This file defines the includes and data structures used by the lingerd
 * daemon.
 */

#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <syslog.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/uio.h>
#include <sys/poll.h>

/* The struct that holds all the information on a socket */
typedef struct {
  /* Function to call when there's read activity on this socket */
  void (*callback)(int n, int fd);

  /* File descriptor for the socket */
  int fd;

  /*
   * The following fields are valid only for fd_info's that correspond
   * to lingering sockets; they are set to 0 for the listening socket and
   * for the Unix-domain sockets of lingerd's clients.
   */

  /* Time when the socket was added */
  time_t insert_time;

  /* 
   * Expected time to kill the socket (incremented everytime there's
   * something to read) 
   */
  time_t linger_time;

  /* 
   * Kill the socket at this time at the very latest (set when we get
   * the socket
   */
  time_t linger_max;

  /* 
   * Some padding to make the size of this struct a power of 2; this
   * should work on all 32bit machines, and on 64bit ones where time_t,
   * pointers and longs are 64 bits, and where an int in the middle of
   * 64bit data is aligned to 64 bits too.
   */
  long pad1, pad2, pad3;

} fd_info;

typedef struct pollfd poll_entry;

#define	NOW()	(now.tv_sec)

/* 
 * Pre-declare a few functions that are referenced before their
 * definition 
 */
void listen_callback(int, int); 
void linger_callback(int, int);
void cleanup_exit(int);

